﻿Public Class SharedSerialClass
    Inherits System.IO.Ports.SerialPort
    Event TerminalData(ByVal sender As Object, ByRef Text As String)

    Private Const DefaultTimeout As Long = 200
    Private Timeout As Long = DefaultTimeout
    Private mutex_bit As New Object

    Public Sub New()
        MyBase.New()
        DataReceivedHandeled()
    End Sub

    Public Sub DataReceivedHandeled()
        AddHandler Me.DataReceived, AddressOf SerialCallbackTerminal
    End Sub

    Public Sub DataReceivedNotHandeled()
        RemoveHandler Me.DataReceived, AddressOf SerialCallbackTerminal
    End Sub

    Private Sub SerialCallbackTerminal(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs)
        Dim Text As String = ""

        Text = Me.ReadMutex()
        Text = Text.Replace(vbCr, vbCrLf)

        RaiseEvent TerminalData(Me, Text)
    End Sub

    Public Sub SendTextToTerminal(ByRef Text As String)
        RaiseEvent TerminalData(Me, Text)
    End Sub

    Public Overloads Sub Close()
        SyncLock mutex_bit
            If Me.isOpen Then
                MyBase.Close()
            End If
        End SyncLock
    End Sub

    Public Shadows Function isOpen()
        Dim status As Boolean
        SyncLock mutex_bit
            status = MyBase.IsOpen
        End SyncLock
        Return status
    End Function

    Public Sub ProtocolTimeoutDefault()
        Timeout = DefaultTimeout
    End Sub
    Public Sub ProtocolTimeout(ByVal timeout_ms As Long)
        Timeout = timeout_ms
    End Sub

    Public Sub WriteCr(ByVal Query As String)
        Me.DiscardInBuffer()
        Me.Write(Query + vbCr)
    End Sub

    Public Function WriteReadLineCr(ByVal Query As String) As String
        WriteReadLineCr = WriteReadLine(Query + vbCr, Timeout)
    End Function

    Public Function WriteReadLine(ByVal Query As String) As String
        WriteReadLine = WriteReadLine(Query, Timeout)
    End Function

    Public Function WriteReadLine(ByVal Query As String, ByVal SleepTime_ms As Integer) As String
        Const DisplayDebugMessages As Boolean = False
        Dim start As DateTime
        Dim delta_ms As Integer

        SyncLock mutex_bit 'GetType(SharedSerialPort)
            Me.DiscardInBuffer()
            Me.Write(Query)

            If DisplayDebugMessages = True Then Debug.Print("=>" + Query.Replace(Chr(13), "<") + "=")
            'System.Threading.Thread.Sleep(SleepTime_ms)

            WriteReadLine = ""
            start = Now

            Dim time As New TimeSpan
            time = Now - start

            Do
                While Me.BytesToRead > 0 And WriteReadLine.Contains(vbCr) = False
                    'WriteReadLine += Me.ReadExisting()
                    WriteReadLine += Chr(Me.ReadByte())
                    'Stop
                End While
                time = Now - start
                delta_ms = (time.Seconds * 1000) + time.Milliseconds
                If DisplayDebugMessages = True Then Debug.Print(delta_ms.ToString)
            Loop While delta_ms < SleepTime_ms And WriteReadLine.Contains(vbCr) = False
            WriteReadLine = WriteReadLine.Replace(vbCr, "")
            WriteReadLine = WriteReadLine.Replace(vbLf, "")
            WriteReadLine += ""

            If DisplayDebugMessages = True Then Debug.Print("<=" + WriteReadLine + "=>")
            If DisplayDebugMessages = True Then Debug.Print("")

            'If WriteReadLine = "" Then
            '    MsgBox("Command timeout at " + SleepTime_ms.ToString + " ms: " + vbCrLf + vbCrLf + Query)
            'End If

        End SyncLock
    End Function

    Public Function ReadMutex() As String
        Const DisplayDebugMessages As Boolean = False

        ReadMutex = ""
        SyncLock mutex_bit 'GetType(SharedSerialPort)

            If Me.BytesToRead > 0 Then
                ReadMutex += Me.ReadExisting
            End If

            If DisplayDebugMessages = True Then Debug.Print("<=" + ReadMutex + "=>")
            If DisplayDebugMessages = True Then Debug.Print("")
        End SyncLock
        Return ReadMutex
    End Function

    Public Shared Function MakeWhiteSpaceVisable(ByRef WhiteString As String) As String
        MakeWhiteSpaceVisable = ""
        Dim c As Char
        For Each c In WhiteString
            If c < " " Then
                MakeWhiteSpaceVisable += "<" + AscW(c).ToString() + ">"
            Else
                MakeWhiteSpaceVisable += c
            End If
        Next
    End Function

    Private Function ConvertBytesToUInt32(ByRef hexArray() As Byte, ByVal position As Integer) As UInt32
        Try
            Dim iInputIndex As Integer = 0
            Dim iOutputIndex As Integer = 0
            Dim bArray(3) As Byte

            For iInputIndex = position To position + 3
                bArray(iOutputIndex) = hexArray(iInputIndex)
                iOutputIndex += 1
            Next

            Array.Reverse(bArray)

            Return BitConverter.ToUInt32(bArray, 0)
        Catch ex As Exception
            Throw New FormatException("The supplied hex value is either empty or in an incorrect format. Use the following format: 00000000", ex)
        End Try
    End Function

End Class
